#!/bin/bash
#
#	$Id: IPN.gate.sh,v 1.3 1998/03/04 02:04:18 moritz Exp $
#		(C) 1996,97 DFG/M.Watermann, Hannover, FRG
#
#		ZConnect-Gateway fuer Mail.IPN.De -> IPN-B.De
#
#-----------------------------------------------------------------------------
set -h -p

#	konfigurierbare Variablen:
#
# Aufbewahrungs-Zeit fuer PMs:
BackupTage=3
#
# Name des 'benachbarten' Zerb-Systemes:
ZerbSystem="IPN-B.De"
#
# Name der Datei mit weiteren Konfigurations-Angaben:
KonfigDat="/home/helpers/mbox/zconnect/gate.vars"

#
#-----------------------------------------------------------------------------
#	(you shouldn't change anything beyond this point!)
#
#	$Log: IPN.gate.sh,v $
#	Revision 1.3  1998/03/04 02:04:18  moritz
#	neue Version
#	
#	Revision 1.13  1997/11/22 19:32:24  matthias
#	Konfigurations-Datei konfigurierbar gemacht; mw
#
#	Revision 1.12  1996/09/20 16:11:08  matthias
#	Code durchgesehen wg. unix-connect v.074b4MB04
#
#	Revision 1.11  1996/09/11 18:53:40  matthias
#	Code ordentlich aufgeraeumt;
#	Variable 'uuRSmtp' integriert;
#	Crontab-Beispiel am Ende angehaengt;
#	Dateinamen-Erzeugung fuer NW-Server auf Sekundenbasis umgestellt;
#
#	Revision 1.10  1996/09/09 19:18:00  matthias
#	absolute Pfad-Angaben bei System-Befehlen entfernt;
#	Modifikationen zur Unterscheidung zwischen 'Barbara' und 'Uluru';
#	globale Variablen R/O gemacht;
#
#	Revision 1.9  1996/08/13 14:17:51  matthias
#	Fehler in einer Rechen-Operation korrigiert;
#
#	Revision 1.8  1996/08/10 23:36:47  matthias
#	Variable "Verbose" steuert Erzeugung von Copy-Meldungen;
#	externe Befehle mit komplettem Pfad angegeben;
#	zusaetzliche Flag-Datei zur Kommunikation mit "Buchen.sh";
#	auch alte Buchungs-Daten werden in "DelBackups" geloescht;
#
#	Revision 1.7  1996/08/09 16:55:52  matthias
#	einige Variablen nach "gate.vars" ausgelagert; 
#
#	Revision 1.6  1996/08/08 18:24:33  matthias
#	Workflag integriert;
#
#	Revision 1.5  1996/08/05 21:43:43  matthias
#	generelles 'umask 007' bei Datei-Erzeugung;
#
#	Revision 1.4  1996/08/04 15:02:51  matthias
#	Typo beim Datums-Format korrigiert;
#
#	Revision 1.3  1996/08/03 23:11:27  matthias
#	SystemZaehler in 'Lin2Zerb' ergaenzt;
#
#	Revision 1.2  1996/08/02 17:38:47  matthias
#	automatisches Loeschen alter PMs integriert;
#
#	Revision 1.1  1996/07/30 11:28:08  matthias
#	Initial revision
#	nach Anregungen der UnixConnect-Scripts;
#	wird regelmaessig via CronD aufgerufen
#	(wg. chown/chmod und SMail-Zugriff als "root");
#
#-----------------------------------------------------------------------------
#	interne Arbeitsvariablen
KonfigDat="${KonfigDat:-/home/helpers/mbox/zconnect/gate.vars}"
[ -s ${KonfigDat} ] || exit 1
. ${KonfigDat}					# Gateway-Variablen einlesen
declare -r PreArcDir=${Me}.${MyPID}
declare -r Host="`hostname -s`"
declare -r PufferEXT="puf"
#
#-----------------------------------------------------------------------------
#
function Einpacken() {
#
#	Aufruf: Einpacken PufferName ZcDateien
#
#	haengt die als $2...$x uebergebenen Dateien an den Puffer $1
#
local Puffer=${1}.${PufferEXT}
  shift ; [ $# -lt 1 ] && return 1
  cat $* >> ${Puffer} 2>/dev/null
  if [ -s ${Puffer} ]; then
    chown ${Owner} ${Puffer}
  else
    rm -f ${Puffer}
  fi
  rm -f $*
  return 0
} # Einpacken

#-----------------------------------------------------------------------------
#
function Importieren(){
#
#	Aufruf: Importieren SysName PufferName PufferExtension
#
local BackInFile=${BackInDir}/${Date}.`date +%s`
local Sys=${1} ; local File=${2} ; local Ext=${3}
local RSMTP="smail -bm -bS -oMs ${Sys} -oMr zconnect"
  [ $# -lt 3 ] && return 1
  shift ; shift ; shift
  [ -s ${File} ] || return		# leere Dateien werden ignoriert
  cp -f ${File} ${BackInFile}.${Ext}
  case ${Ext} in
    "prv"|"PRV"|\
    "eil"|"EIL" )
	${BinDir}/uuwsmtp -f ${File} ${Sys} | ${RSMTP}
	;;
    "brt"|"BRT" )
	${BinDir}/uuwnews -f ${File} ${Sys} | rnews
	;;
  esac
  mv -f ${File} ${BackInFile}.${Ext} &>/dev/null
  let SomethingDone+=1
  [ -s ${BackInFile}.${Ext} ] && chown ${Owner} ${BackInFile}.${Ext}
} # Importieren

#-----------------------------------------------------------------------------
#
function DelBackups() {
#
#	loescht alte PRV-Backups
#
local Dir=${1}
  [ \( $# -gt 0 \) -a \( -d ${Dir} \) ] || return 1
  find ${Dir}/*.prv ${Dir}/*.dat -atime +${BackupTage} -exec rm -f {} \; 2> /dev/null
  return 0
} # DelBackups

#-----------------------------------------------------------------------------
#
function Lin2Zerb() {
#
#	Routine zum Verschieben der ZConnect-Puffer auf den Netware-Server
#
local LClientFlag="${Lin2ZerbDIR}/${ClientFlag}"
local LMasterFlag="${Lin2ZerbDIR}/${MasterFlag}"
local LPuffer ; local P ; local E ; local Secs
local Exts="prv brt" ; local Sys
  DelBackups ${BackOutDir}
  for Sys in ${SystemsDir}/* ; do
    Sys=${Sys##*/}
    if [ -d ${Spool}/${Sys} ] ; then
      builtin cd ${Spool}/${Sys}	# z.B.: /var/spool/zconnect/ipn-b.de
      # System nur bearbeiten, wenn Arbeit vorhanden ist
      P="`find . -type f -print`"
      if [ -n "${P}" ] ; then
	mkdir -p ${PreArcDir}
	mv -f ${P} ${PreArcDir} &>/dev/null
	builtin cd ${PreArcDir}
	Einpacken ${Spool}/prv.${Sys} *.prv *.eil *.neu
	Einpacken ${Spool}/brt.${Sys} *.brt
	mv -f * ../ &>/dev/null		# ggf. uebriggebliebenes zurueckschieben
	builtin cd ..
	rm -rf ${PreArcDir}
      fi
    fi
  done
  [ -d ${Lin2ZerbDIR} ] || return	# Netware-Dir nicht verfuegbar
  [ -f ${LMasterFlag} ] && return	# wird vom Zerb-Cronjob erzeugt
  echo ${MyPID} >${LClientFlag}		# wird vom Zerb-Cronjob respektiert
  [ -w ${LClientFlag} ] || return
  for E in ${Exts}; do
    for P in ${Spool}/${E}.*.${PufferEXT}; do
      if [ -s ${P} ] ; then
	Secs=`date +%s`
	LPuffer="${BackOutDir}/${Date}.${Secs}.${E} "
	mv -f${Verbose} ${P} ${LPuffer}
	chown ${Owner} ${LPuffer}
	while [ ${#Secs} -gt 8 ] ; do	# auf 8.3-Konvention (DOS) kuerzen
	  Secs=${Secs#?}
	done
	cp -f${Verbose} ${LPuffer} ${Lin2ZerbDIR}/${Secs}.${E}
	let SomethingDone+=1
	sleep 1				# zur Sicherheit eine Sekunde ausruhen
      else
         rm -f ${P}
      fi
    done
  done
  rm -f ${LClientFlag}
  return 0
} # Lin2Zerb

#-----------------------------------------------------------------------------
#
function Zerb2Lin() {
#
#	Routine zum Holen der ZConnect-Puffer vom Netware-Server
#
local ZClientFlag="${Zerb2LinDIR}/${ClientFlag}"
local ZMasterFlag="${Zerb2LinDIR}/${MasterFlag}"
local ZPuffer ; local P ; local E
local Exts="prv brt eil"
  DelBackups ${BackInDir}
  [ -d ${Zerb2LinDIR} ] || return	# Netware-Dir nicht verfuegbar
  [ -f ${ZMasterFlag} ] && return	# wird vom Zerb-Cronjob erzeugt
  echo ${MyPID} >${ZClientFlag}		# wird vom Zerb-Cronjob respektiert
  [ -w ${ZClientFlag} ] || return
  for E in ${Exts}; do
    for P in ${Zerb2LinDIR}/*.${E}; do
      if [ -s ${P} ] ; then
	ZPuffer=${Spool}/`date +%s`.${E}
        cp -f${Verbose} ${P} ${ZPuffer}
	if [ -s ${ZPuffer} ] ; then
	  rm -f ${P}
	  chown ${Owner} ${ZPuffer} 2>/dev/null
	  chmod -x ${ZPuffer}
	  Importieren ${ZerbSystem} ${ZPuffer} ${E}
	  let SomethingDone+=1
	  sleep 1			# zur Sicherheit eine Sekunde ausruhen
	else
           rm -f ${ZPuffer}
	fi
      fi
    done
  done
  rm -f ${ZClientFlag}
  return 0
} # Zerb2Lin

#----------------------------------------------------------------------------
#
# Main()
#
builtin cd ${BinDir} || exit 1		# ins Home-Verzeichnis wechseln
[ -f ${WorkFlag} ] && exit 1		# andere Programm-Kopie aktiv?
ChkBothFlag || exit 1			# Buchungs-Script aktiv?

trap " rm -f ${Flag4Both} ${WorkFlag} ; exit " 0 1 2 3 9 15

umask 007 ; echo $$ >${WorkFlag}
[ -w ${WorkFlag} ] || exit 1		# Arbeits-Flag OK?

[ "${Host}" = "Barbara" ] && ZerbSystem="zgate"	# fuer Matthias

[ -z "${uuRSmtp}" ] && ${BinDir}/zbatchsmtp	# SMTP-Mails verarbeiten

Lin2Zerb				# Mails/News auf NW-Server schieben

Zerb2Lin				# Mails/News vom NW-Server holen

[ -n "${Verbose}" -a ${SomethingDone} -gt 0 ] && times

exit 0

#-----------------------------------------------------------------------------
#
# Das Script ist hier in '/etc/crontab' so eingebunden:
#
# uucp- und ZConnect-Nachrichten konvertieren:
7-55/6   0  * * * root /home/helpers/mbox/zconnect/IPN.gate.sh
1-55/6 1-22 * * * root /home/helpers/mbox/zconnect/IPN.gate.sh
1-49/6  23  * * * root /home/helpers/mbox/zconnect/IPN.gate.sh
#
# _EoF_
