#! /bin/sh
# $Id: auspack.SH,v 1.6 2000/06/20 13:27:32 moritz Exp $
#
#   auspack     generisches Ent-Archivierungs-Kommando fuer ZCONNECT
#
#   Aufruf:     auspack (Kennung) (Archiv) 
#
#   Funktion:   Extrahiert alle Dateien (ohne Directory) aus dem angegebenen
#               Archiv in das aktuelle Verzeichnis.
#
#               (Kennung)   der ZCONNECT-Code fuer den verwendeten Arcer,
#                           also z.B. ZIP ARC ZOO LHA NONE
#               (Archiv)    die gepackte Datei mit oder ohne Kennung
#
#   Es sollte keine Meldungen ausgegeben werden. Directories werden nicht
#   mit ausgepackt.
#
#   Wenn als dritter Parameter (filename) filename angegeben wird, wird
#   eine Zeile auf stdout ausgegeben, die den Namen der entpackten Datei
#   angibt.

if [ "x$2" = "x" ] 
then
	echo "Aufruf:  $0 (Kennung) (Archiv)" >&2
	exit 1
fi

KENNUNG=$1
ARC=$2

shift; shift

FILENAME=no
if [ "$1" = "filename" ]; then
	FILENAME=yes
fi

case "$KENNUNG" in
 "zip" | \
 "ZIP" )	name=`dirname $ARC`/`basename $ARC .zip`
		if [ -r $name.zip ]
		then
			ARC="$name".zip
		fi
		unzip $ARC	> /dev/null 2>&1 
		EC=$?
		;;
 "arc" | \
 "ARC" )	name=`dirname $ARC`/`basename $ARC .arc`
		if [ -r $name.arc ]
		then
			ARC="$name".arc
		fi
		arc x $ARC	> /dev/null 2>&1
		EC=$?
		;;
 "zoo" | \
 "ZOO" )	name=`dirname $ARC`/`basename $ARC .zoo`
		if [ -r $name.zoo ]
		then
			ARC="$name".zoo
		fi
		zoo x $ARC	> /dev/null 2>&1
		EC=$?
		;;
 "lha" | \
 "LHA" )	name=`dirname $ARC`/`basename $ARC .lha`
		if [ -r $name.lha ]
		then
			ARC="$name".lha
		fi
		if [ ! -r $ARC ] 
		then
			name=`dirname $ARC`/`basename $ARC .lha`
			if [ -r $name.lzh ]
			then
				ARC="$name".lzh
			fi
		fi
		lha xi $ARC	> /dev/null 2>&1
		EC=$?
		;;
 "none" | \
 "NONE")	name=`dirname $ARC`/`basename $ARC .nar`
		if [ -r $name.nar ]
		then
			ARC="$name".nar
		fi
		(
			cp $ARC puffer$$ || exit 1
			zconsplit puffer$$
		)
		EC=$?
		;;
 *)		mail postmaster << EOF
ZCONNECT Warnung

Achtung:

	Aufruf von 'auspack' mit illegalen Parametern: $KENNUNG $ARC $*
EOF
		EC=1
		;;
esac

test "$FILENAME" = "yes" && echo "$ARC"

exit $EC

